@@ -17,18 +17,20 @@
 # You should have received a copy of the GNU Affero General Public License
 # along with weboob. If not, see <http://www.gnu.org/licenses/>.
 
-
-from weboob.capabilities.base import NotAvailable, NotLoaded
+from weboob.capabilities.calendar import BaseCalendarEvent, TRANSP, STATUS, CATEGORIES
+from weboob.capabilities.collection import Collection
+from weboob.capabilities.video import BaseVideo
+from weboob.capabilities.image import BaseImage
+from weboob.capabilities.base import NotAvailable, NotLoaded, find_object
 from weboob.capabilities.cinema import Movie, Person
 from weboob.deprecated.browser import Browser
 from weboob.tools.json import json
 import base64
 import hashlib
-from datetime import datetime
+from datetime import datetime, date, timedelta
 import time
 import urllib
 
-
 __all__ = ['AllocineBrowser']
 
 
@@ -53,7 +55,7 @@ class AllocineBrowser(Browser):
 
     def iter_movies(self, pattern):
         params = [('partner', self.PARTNER_KEY),
-                  ('q', pattern.encode('utf-8')),
+                  ('q', pattern),
                   ('format', 'json'),
                   ('filter', 'movie')]
 
@@ -92,7 +94,7 @@ class AllocineBrowser(Browser):
 
     def iter_persons(self, pattern):
         params = [('partner', self.PARTNER_KEY),
-                  ('q', pattern.encode('utf-8')),
+                  ('q', pattern),
                   ('format', 'json'),
                   ('filter', 'person')]
 
@@ -185,12 +187,13 @@ class AllocineBrowser(Browser):
         if 'synopsis' in jres:
             pitch = unicode(jres['synopsis'])
         if 'statistics' in jres and 'userRating' in jres['statistics']:
-            note = u'%s/10 (%s votes)' % (jres['statistics']['userRating'], jres['statistics']['userReviewCount'])
+            note = u'%s/5 (%s votes)' % (jres['statistics']['userRating'], jres['statistics']['userReviewCount'])
         if 'castMember' in jres:
             for cast in jres['castMember']:
                 if cast['activity']['$'] not in roles:
                     roles[cast['activity']['$']] = []
-                roles[cast['activity']['$']].append(cast['person']['name'])
+                person_to_append = (u'%s' % cast['person']['code'], cast['person']['name'])
+                roles[cast['activity']['$']].append(person_to_append)
 
         movie = Movie(id, title)
         movie.other_titles = other_titles
@@ -277,8 +280,8 @@ class AllocineBrowser(Browser):
                 pyear = '????'
                 if 'productionYear' in m['movie']:
                     pyear = m['movie']['productionYear']
-                roles[m['activity']['$']].append(u'(%s) %s' % (pyear, m['movie']['originalTitle']))
-
+                movie_to_append = (u'%s' % (m['movie']['code']), u'(%s) %s' % (pyear, m['movie']['originalTitle']))
+                roles[m['activity']['$']].append(movie_to_append)
 
         person = Person(id, name)
         person.real_name = real_name
@@ -441,3 +444,206 @@ class AllocineBrowser(Browser):
             biography = unicode(jres['biography'])
 
         return biography
+
+    def get_categories_movies(self, category):
+        params = [('partner', self.PARTNER_KEY),
+                  ('format', 'json'),
+                  ('mediafmt', 'mp4'),
+                  ('filter', category)
+                  ]
+        res = self.__do_request('movielist', params)
+        if res is None:
+            return
+        result = json.loads(res)
+        for movie in result['feed']['movie']:
+            if 'trailer' not in movie or 'productionYear' not in movie:
+                continue
+            yield self.parse_movie(movie)
+
+    def get_categories_videos(self, category):
+        params = [('partner', self.PARTNER_KEY),
+                  ('format', 'json'),
+                  ('mediafmt', 'mp4'),
+                  ('filter', category)
+                  ]
+        res = self.__do_request('videolist', params)
+        if res is None:
+            return
+        result = json.loads(res)
+        if 'feed' in result and 'media' in result['feed']:
+            for episode in result['feed']['media']:
+                if 'title' in episode:
+                    yield self.parse_video(episode, category)
+
+    def parse_video(self, _video, category):
+        video = BaseVideo(u'%s#%s' % (_video['code'], category))
+        video.title = unicode(_video['title'])
+        video._video_code = unicode(_video['code'])
+        video.ext = u'mp4'
+        if 'runtime' in _video:
+            video.duration = timedelta(seconds=int(_video['runtime']))
+        if 'description' in _video:
+            video.description = unicode(_video['description'])
+        renditions = sorted(_video['rendition'], key=lambda x: 'bandwidth' in x and x['bandwidth']['code'], reverse=True)
+        video.url = unicode(max(renditions, key=lambda x: 'bandwidth' in x)['href'])
+        return video
+
+    def parse_movie(self, movie):
+        video = BaseVideo(u'%s#%s' % (movie['code'], 'movie'))
+        video.title = unicode(movie['trailer']['name'])
+        video._video_code = unicode(movie['trailer']['code'])
+        video.ext = u'mp4'
+        if 'poster' in movie:
+            video.thumbnail = BaseImage(movie['poster']['href'])
+            video.thumbnail.url = unicode(movie['poster']['href'])
+        tdate = movie['release']['releaseDate'].split('-')
+        day = 1
+        month = 1
+        year = 1901
+        if len(tdate) > 2:
+            year = int(tdate[0])
+            month = int(tdate[1])
+            day = int(tdate[2])
+
+        video.date = date(year, month, day)
+        if 'userRating' in movie['statistics']:
+            video.rating = movie['statistics']['userRating']
+        elif 'pressRating' in movie['statistics']:
+            video.rating = movie['statistics']['pressRating'] * 2
+        video.rating_max = 5
+        if 'synopsis' in movie:
+            video.description = unicode(movie['synopsis'].replace('<p>', '').replace('</p>', ''))
+        elif 'synopsisShort' in movie:
+            video.description = unicode(movie['synopsisShort'].replace('<p>', '').replace('</p>', ''))
+        if 'castingShort' in movie:
+            if 'directors' in movie['castingShort']:
+                video.author = unicode(movie['castingShort']['directors'])
+        if 'runtime' in movie:
+            video.duration = timedelta(seconds=int(movie['runtime']))
+        return video
+
+    def get_movie_from_id(self, _id):
+        params = [('partner', self.PARTNER_KEY),
+                  ('format', 'json'),
+                  ('mediafmt', 'mp4'),
+                  ('filter', 'movie'),
+                  ('code', _id),
+                  ]
+        res = self.__do_request('movie', params)
+        if res is None:
+            return
+        result = json.loads(res)
+        return self.parse_video(result['movie'])
+
+    def get_video_from_id(self, _id, category):
+        return find_object(self.get_categories_videos(category), id=u'%s#%s' % (_id, category))
+
+    def get_video_url(self, code):
+        params = [('partner', self.PARTNER_KEY),
+                  ('format', 'json'),
+                  ('mediafmt', 'mp4'),
+                  ('code', code),
+                  ('profile', 'large'),
+                  ]
+        res = self.__do_request('media', params)
+        if res is None:
+            return
+        result = json.loads(res)
+        renditions = sorted(result['media']['rendition'], key=lambda x: 'bandwidth' in x and x['bandwidth']['code'], reverse=True)
+        return max(renditions, key=lambda x: 'bandwidth' in x)['href']
+
+    def get_emissions(self, basename):
+        params = [('partner', self.PARTNER_KEY),
+                  ('format', 'json'),
+                  ('filter', 'acshow'),
+                  ]
+        res = self.__do_request('termlist', params)
+        if res is None:
+            return
+        result = json.loads(res)
+        for emission in result['feed']['term']:
+            yield Collection([basename, unicode(emission['nameShort'])], unicode(emission['$']))
+
+    def search_events(self, query):
+        params = [('partner', self.PARTNER_KEY),
+                  ('format', 'json'),
+                  ('zip', query.city)
+                  ]
+
+        if query.summary:
+            movie = self.iter_movies(query.summary).next()
+            params.append(('movie', movie.id))
+
+        res = self.__do_request('showtimelist', params)
+        if res is None:
+            return
+        result = json.loads(res)
+
+        for event in self.create_event(result):
+            if (not query.end_date or event.start_date <= query.end_date)\
+               and event.start_date >= query.start_date:
+                yield event
+
+    def get_event(self, _id):
+        split_id = _id.split('#')
+        params = [('partner', self.PARTNER_KEY),
+                  ('format', 'json'),
+                  ('theaters', split_id[0]),
+                  ('zip', split_id[1]),
+                  ('movie', split_id[2]),
+                  ]
+
+        res = self.__do_request('showtimelist', params)
+        if res is None:
+            return
+        result = json.loads(res)
+
+        for event in self.create_event(result):
+            if event.id.split('#')[-1] == split_id[-1]:
+                event.description = self.get_movie(split_id[2]).pitch
+                return event
+
+    def create_event(self, data):
+        sequence = 1
+        transp = TRANSP.TRANSPARENT
+        status = STATUS.CONFIRMED
+        category = CATEGORIES.CINE
+
+        if 'theaterShowtimes' not in data['feed']:
+            return
+
+        for items in data['feed']['theaterShowtimes']:
+            cinema = items['place']['theater']
+            city = unicode(cinema['city'])
+            location = u'%s, %s' % (cinema['name'], cinema['address'])
+            postalCode = cinema['postalCode']
+            cinemaCode = cinema['code']
+            for show in items['movieShowtimes']:
+                summary = unicode(show['onShow']['movie']['title'])
+                movieCode = show['onShow']['movie']['code']
+                for jour in show['scr']:
+                    tdate = jour['d'].split('-')
+                    year = int(tdate[0])
+                    month = int(tdate[1])
+                    day = int(tdate[2])
+                    for seance in jour['t']:
+                        ttime = seance['$'].split(':')
+                        heure = int(ttime[0])
+                        minute = int(ttime[1])
+                        start_date = datetime(year=year, month=month, day=day,
+                                              hour=heure, minute=minute)
+
+                        seanceCode = seance['code']
+                        _id = u'%s#%s#%s#%s' % (cinemaCode, postalCode, movieCode, seanceCode)
+                        event = BaseCalendarEvent()
+                        event.id = _id
+                        event.sequence = sequence
+                        event.transp = transp
+                        event.status = status
+                        event.category = category
+                        event.city = city
+                        event.location = location
+                        event.start_date = start_date
+                        event.summary = summary
+                        event.timezone = u'Europe/Paris'
+                        yield event
