@@ -22,11 +22,18 @@ import datetime
 from urlparse import urlsplit, parse_qsl
 from decimal import Decimal
 import re
+import urllib
 from mechanize import Cookie, FormNotFoundError
 
-from weboob.deprecated.browser import Page as _BasePage, BrowserUnavailable, BrokenPageError
-from weboob.capabilities.bank import Account
+
+from weboob.browser.filters.standard import CleanText
+from weboob.exceptions import BrowserUnavailable, BrowserIncorrectPassword
+from weboob.deprecated.browser import Page as _BasePage, BrokenPageError
+from weboob.capabilities.bank import Account, Investment
+from weboob.capabilities import NotAvailable, UserError
 from weboob.tools.capabilities.bank.transactions import FrenchTransaction
+from weboob.tools.json import json
+from weboob.tools.pdf import decompress_pdf
 
 
 class WikipediaARC4(object):
@@ -55,7 +62,64 @@ class WikipediaARC4(object):
 
 class BasePage(_BasePage):
     def get_token(self):
-        return self.parser.select(self.document.getroot(), '//form//input[@name="token"]', 1, 'xpath').attrib['value']
+        try:
+            token = self.parser.select(self.document.getroot(), '//form//input[@name="token"]', 1, 'xpath').attrib['value']
+        except BrokenPageError:
+            token = self.parser.select(self.document.getroot(), '//body', 1, 'xpath').attrib['onload']
+            token = re.search(r"saveToken\('(.*?)'", token).group(1)
+        return token
+
+    def on_loaded(self):
+        if not self.is_error():
+            self.browser.token = self.get_token()
+            self.logger.debug('Update token to %s', self.browser.token)
+
+    def is_error(self):
+        return False
+
+    def build_token(self, token):
+        """
+        These fucking faggots have introduced a new protection on the token.
+
+        Each time there is a call to SAB (selectActionButton), the token
+        available in the form is modified with a key available in JS:
+
+        ipsff(function(){TW().ipthk([12, 25, 17, 5, 23, 26, 15, 30, 6]);});
+
+        Each value of the array is an index for the current token to append the
+        char at this position at the end of the token.
+        """
+        table = None
+        for script in self.document.xpath('//script'):
+            if script.text is None:
+                continue
+            m = re.search(r'ipthk\(([^\)]+)\)', script.text, flags=re.MULTILINE)
+            if m:
+                table = json.loads(m.group(1))
+        if table is None:
+            return token
+
+        for i in table:
+            token += token[i]
+        return token
+
+    def get_params(self):
+        params = {}
+        for field in self.document.xpath('//input'):
+            params[field.attrib['name']] = field.attrib.get('value', '')
+        return params
+
+    def get_button_actions(self):
+        actions = {}
+        for script in self.document.xpath('//script'):
+            if script.text is None:
+                continue
+
+            for id, action, strategy in re.findall(r'''attEvt\(window,"(?P<id>[^"]+)","click","sab\('(?P<action>[^']+)','(?P<strategy>[^']+)'\);"''', script.text, re.MULTILINE):
+                actions[id] = {'dialogActionPerformed': action,
+                               'validationStrategy': strategy,
+                              }
+        return actions
 
 
 class RedirectPage(BasePage):
@@ -141,6 +205,17 @@ class RedirectPage(BasePage):
             self.browser.submit(nologin=True)
 
 
+class ErrorPage(BasePage):
+    def get_token(self):
+        try:
+            buf = self.document.xpath('//body/@onload')[0]
+        except IndexError:
+            return
+        else:
+            m = re.search("saveToken\('([^']+)'\)", buf)
+            if m:
+                return m.group(1)
+
 class UnavailablePage(BasePage):
     def on_loaded(self):
         try:
@@ -148,7 +223,7 @@ class UnavailablePage(BasePage):
         except IndexError:
             raise BrowserUnavailable()
         else:
-            self.browser.location(a.attrib['href'])
+            self.browser.location(a.attrib['href'], nologin=True)
 
 
 class LoginPage(BasePage):
@@ -171,6 +246,50 @@ class LoginPage(BasePage):
         self.browser.submit(nologin=True)
 
 
+class Login2Page(LoginPage):
+    @property
+    def request_url(self):
+        transactionID = self.group_dict['transactionID']
+        return 'https://www.icgauth.banquepopulaire.fr/dacswebssoissuer/api/v1u0/transaction/%s' % transactionID
+
+    def on_loaded(self):
+        r = self.browser.openurl(self.request_url)
+        doc = json.load(r)
+        self.form_id = doc['step']['validationUnits'][0]['PASSWORD_LOOKUP'][0]['id']
+
+    def login(self, login, password):
+        payload = {'validate': {'PASSWORD_LOOKUP': [{'id': self.form_id,
+                                                     'login': login.encode(self.browser.ENCODING).upper(),
+                                                     'password': password.encode(self.browser.ENCODING),
+                                                     'type': 'PASSWORD_LOOKUP'
+                                                    }]
+                               }
+                  }
+        req = self.browser.request_class(self.request_url + '/step')
+        req.add_header('Content-Type', 'application/json')
+        r = self.browser.openurl(req, json.dumps(payload))
+
+        doc = json.load(r)
+        self.logger.debug(doc)
+        if 'phase' in doc and doc['phase']['state'] == 'TERMS_OF_USE':
+            # Got:
+            # {u'phase': {u'state': u'TERMS_OF_USE'}, u'validationUnits': [{u'LIST_OF_TERMS': [{u'type': u'TERMS', u'id': u'b7f28f91-7aa0-48aa-8028-deec13ae341b', u'reference': u'CGU_CYBERPLUS'}]}]}
+            if 'reference' in doc['validationUnits'][0]:
+                del doc['validationUnits'][0]['reference']
+            payload = {'validate': doc['validationUnits'][0]}
+            req = self.browser.request_class(self.request_url + '/step')
+            req.add_header('Content-Type', 'application/json')
+            r = self.browser.openurl(req, json.dumps(payload))
+            doc = json.load(r)
+            self.logger.debug(doc)
+
+        if ('phase' in doc and doc['phase']['previousResult'] == 'FAILED_AUTHENTICATION') or \
+           doc['response']['status'] != 'AUTHENTICATION_SUCCESS':
+            raise BrowserIncorrectPassword()
+
+        self.browser.location(doc['response']['saml2_post']['action'], urllib.urlencode({'SAMLResponse': doc['response']['saml2_post']['samlResponse']}))
+
+
 class IndexPage(BasePage):
     def get_token(self):
         url = self.document.getroot().xpath('//frame[@name="portalHeader"]')[0].attrib['src']
@@ -194,17 +313,16 @@ class HomePage(BasePage):
                     vary = m.group(1)
                     break
 
-        #r = self.browser.openurl(self.browser.request_class(self.browser.buildurl(self.browser.absurl("/portailinternet/_layouts/Ibp.Cyi.Application/GetuserInfo.ashx"), action='UInfo', vary=vary), None, {'Referer': self.url}))
-        #print r.read()
-
         r = self.browser.openurl(self.browser.request_class(self.browser.buildurl(self.browser.absurl('/portailinternet/Transactionnel/Pages/CyberIntegrationPage.aspx'), vary=vary), 'taskId=aUniversMesComptes', {'Referer': self.url}))
+        if not int(r.info().get('Content-Length', '')):
+            r = self.browser.openurl(self.browser.request_class(self.browser.buildurl(self.browser.absurl('/portailinternet/Transactionnel/Pages/CyberIntegrationPage.aspx')), 'taskId=aUniversMesComptes', {'Referer': self.url}))
         doc = self.browser.get_document(r)
         date = None
         for script in doc.xpath('//script'):
             if script.text is None:
                 continue
 
-            m = re.search('lastConnectionDate":"([^"]+)"', script.text)
+            m = re.search('lastConnectionDate":"([^"]*)"', script.text)
             if m:
                 date = m.group(1)
 
@@ -215,14 +333,19 @@ class HomePage(BasePage):
 
 
 class AccountsPage(BasePage):
-    ACCOUNT_TYPES = {u'Mes comptes d\'épargne':     Account.TYPE_SAVINGS,
-                     u'Mon épargne':                Account.TYPE_SAVINGS,
-                     u'Placements':                 Account.TYPE_SAVINGS,
-                     u'Mes comptes':                Account.TYPE_CHECKING,
-                     u'Comptes en euros':           Account.TYPE_CHECKING,
-                     u'Mes emprunts':               Account.TYPE_LOAN,
-                     u'Financements':               Account.TYPE_LOAN,
-                     u'Mes services':               None,    # ignore this kind of accounts (no bank ones)
+    ACCOUNT_TYPES = {u'Mes comptes d\'épargne':        Account.TYPE_SAVINGS,
+                     u'Mon épargne':                   Account.TYPE_SAVINGS,
+                     u'Placements':                    Account.TYPE_SAVINGS,
+                     u'Liste complète de mon épargne': Account.TYPE_SAVINGS,
+                     u'Mes comptes':                   Account.TYPE_CHECKING,
+                     u'Comptes en euros':              Account.TYPE_CHECKING,
+                     u'Liste complète de mes comptes': Account.TYPE_CHECKING,
+                     u'Mes emprunts':                  Account.TYPE_LOAN,
+                     u'Liste complète de mes emprunts':Account.TYPE_LOAN,
+                     u'Financements':                  Account.TYPE_LOAN,
+                     u'Mes services':                  None,    # ignore this kind of accounts (no bank ones)
+                     u'Équipements':                   None,    # ignore this kind of accounts (no bank ones)
+                     u'Synthèse':                      None,    # ignore this title
                     }
 
     def is_error(self):
@@ -234,6 +357,11 @@ class AccountsPage(BasePage):
 
         return False
 
+    def pop_up(self):
+        if self.document.xpath('//span[contains(text(), "du navigateur Internet.")]'):
+            return True
+        return False
+
     def is_short_list(self):
         return len(self.document.xpath('//script[contains(text(), "EQUIPEMENT_COMPLET")]')) > 0
 
@@ -246,17 +374,26 @@ class AccountsPage(BasePage):
     def iter_accounts(self, next_pages):
         account_type = Account.TYPE_UNKNOWN
 
-        params = {}
-        for field in self.document.xpath('//input'):
-            params[field.attrib['name']] = field.attrib.get('value', '')
+        params = self.get_params()
+        actions = self.get_button_actions()
 
         for div in self.document.getroot().cssselect('div.btit'):
-            if div.text is None:
+            if div.text in (None, u'Synthèse'):
                 continue
             account_type = self.ACCOUNT_TYPES.get(div.text.strip(), Account.TYPE_UNKNOWN)
 
             if account_type is None:
                 # ignore services accounts
+                self.logger.debug('Ignore account type %s', div.text.strip())
+                continue
+
+            # Go to the full list of this kind of account, if any.
+            btn = div.getparent().xpath('.//button/span[text()="Suite"]')
+            if len(btn) > 0:
+                btn = btn[0].getparent()
+                _params = params.copy()
+                _params.update(actions[btn.attrib['id']])
+                next_pages.append(_params)
                 continue
 
             currency = None
@@ -292,6 +429,7 @@ class AccountsPage(BasePage):
                 account._next_debit = None
                 account._params = None
                 account._coming_params = None
+                account._invest_params = None
                 if balance != u'' and len(tds[3].xpath('.//a')) > 0:
                     account._params = params.copy()
                     account._params['dialogActionPerformed'] = 'SOLDE'
@@ -300,9 +438,30 @@ class AccountsPage(BasePage):
                 if len(tds) >= 5 and len(tds[self.COL_COMING].xpath('.//a')) > 0:
                     _params = account._params.copy()
                     _params['dialogActionPerformed'] = 'ENCOURS_COMPTE'
+
+                    # If there is an action needed before going to the cards page, save it.
+                    m = re.search('dialogActionPerformed=([\w_]+)', self.url)
+                    if m and m.group(1) != 'EQUIPEMENT_COMPLET':
+                        _params['prevAction'] = m.group(1)
                     next_pages.append(_params)
+
+                if not account._params:
+                    account._invest_params = params.copy()
+                    account._invest_params['dialogActionPerformed'] = 'CONTRAT'
+                    account._invest_params['attribute($SEL_$%s)' % tr.attrib['id'].split('_')[0]] = tr.attrib['id'].split('_', 1)[1]
+
                 yield account
 
+        # Needed to preserve navigation.
+        btn = self.document.xpath('.//button/span[text()="Retour"]')
+        if len(btn) > 0:
+            btn = btn[0].getparent()
+            _params = params.copy()
+            _params.update(actions[btn.attrib['id']])
+            self.browser.openurl('/cyber/internet/ContinueTask.do', urllib.urlencode(_params))
+
+class AccountsFullPage(AccountsPage):
+    pass
 
 class CardsPage(BasePage):
     COL_ID = 0
@@ -311,10 +470,8 @@ class CardsPage(BasePage):
     COL_DATE = 3
     COL_AMOUNT = 4
 
-    def iter_accounts(self):
-        params = {}
-        for field in self.document.xpath('//input'):
-            params[field.attrib['name']] = field.attrib.get('value', '')
+    def iter_accounts(self, next_pages):
+        params = self.get_params()
 
         account = None
         for tr in self.document.xpath('//table[@id="TabCtes"]/tbody/tr'):
@@ -326,11 +483,14 @@ class CardsPage(BasePage):
                     yield account
                 account = Account()
                 account.id = id.replace(' ', '')
+                account.type = Account.TYPE_CARD
                 account.balance = account.coming = Decimal('0')
                 account._next_debit = datetime.date.today()
                 account._prev_debit = datetime.date(2000,1,1)
                 account.label = u' '.join([self.parser.tocleanstring(cols[self.COL_TYPE]),
                                            self.parser.tocleanstring(cols[self.COL_LABEL])])
+                account._params = None
+                account._invest_params = None
                 account._coming_params = params.copy()
                 account._coming_params['dialogActionPerformed'] = 'SELECTION_ENCOURS_CARTE'
                 account._coming_params['attribute($SEL_$%s)' % tr.attrib['id'].split('_')[0]] = tr.attrib['id'].split('_', 1)[1]
@@ -364,6 +524,15 @@ class CardsPage(BasePage):
         if account is not None:
             yield account
 
+        # Needed to preserve navigation.
+        btn = self.document.xpath('.//button/span[text()="Retour"]')
+        if len(btn) > 0:
+            btn = btn[0].getparent()
+            actions = self.get_button_actions()
+            _params = params.copy()
+            _params.update(actions[btn.attrib['id']])
+            self.browser.openurl('/cyber/internet/ContinueTask.do', urllib.urlencode(_params))
+
 
 class Transaction(FrenchTransaction):
     PATTERNS = [(re.compile('^RET DAB (?P<text>.*?) RETRAIT (DU|LE) (?P<dd>\d{2})(?P<mm>\d{2})(?P<yy>\d+).*'),
@@ -432,7 +601,7 @@ class TransactionsPage(BasePage):
             if len(tds) < 5:
                 continue
 
-            t = Transaction(tr.attrib.get('id', '0_0').split('_', 1)[1])
+            t = Transaction()
 
             # XXX We currently take the *value* date, but it will probably
             # necessary to use the *operation* one.
@@ -463,10 +632,12 @@ class TransactionsPage(BasePage):
     def get_card_history(self, account, coming):
         if coming:
             debit_date = account._next_debit
+        elif not hasattr(account, '_prev_balance'):
+            return
         else:
             debit_date = account._prev_debit
             if 'ContinueTask.do' in self.url:
-                t = Transaction(0)
+                t = Transaction()
                 t.parse(debit_date, 'RELEVE CARTE')
                 t.amount = -account._prev_balance
                 yield t
@@ -477,7 +648,7 @@ class TransactionsPage(BasePage):
             if len(tds) < 3:
                 continue
 
-            t = Transaction(i)
+            t = Transaction()
 
             date = self.parser.tocleanstring(tds[self.COL_CARD_DATE])
             label = self.parser.tocleanstring(tds[self.COL_CARD_LABEL])
@@ -495,3 +666,135 @@ class TransactionsPage(BasePage):
             return True
 
         return False
+
+    def get_investment_page_params(self):
+        script = self.document.xpath('//body')[0].attrib['onload']
+        url = None
+        m = re.search(r"','(.+?)',\[", script, re.MULTILINE)
+        if m:
+            url = m.group(1)
+        params = {}
+        for key, value in re.findall(r"key:'(?P<key>SJRToken)'\,value:'(?P<value>.*?)'}", script, re.MULTILINE):
+            params[key] = value
+        return url, params if url and params else None
+
+
+class LineboursePage(_BasePage):
+    pass
+
+
+class InvestmentLineboursePage(_BasePage):
+    COL_LABEL = 0
+    COL_QUANTITY = 1
+    COL_UNITVALUE = 2
+    COL_VALUATION = 3
+    COL_UNITPRICE = 4
+    COL_PERF_PERCENT = 5
+    COL_PERF = 6
+    def get_investments(self):
+        for line in self.document.xpath('//table[contains(@summary, "Contenu")]/tbody/tr[@class="color4"]'):
+            cols1 = line.findall('td')
+            cols2 = line.xpath('./following-sibling::tr')[0].findall('td')
+
+            inv = Investment()
+            inv.label = self.parser.tocleanstring(cols1[self.COL_LABEL].xpath('.//span')[0])
+            inv.code = self.parser.tocleanstring(cols1[self.COL_LABEL].xpath('./a')[0]).split(' ')[-1]
+            inv.quantity = self.parse_decimal(cols2[self.COL_QUANTITY])
+            inv.unitprice = self.parse_decimal(cols2[self.COL_UNITPRICE])
+            inv.unitvalue = self.parse_decimal(cols2[self.COL_UNITVALUE])
+            inv.valuation = self.parse_decimal(cols2[self.COL_VALUATION])
+            inv.diff = self.parse_decimal(cols2[self.COL_PERF])
+
+            yield inv
+
+    def parse_decimal(self, string):
+        value = self.parser.tocleanstring(string)
+        if value == '':
+            return NotAvailable
+        return Decimal(Transaction.clean_amount(value))
+
+
+class NatixisPage(_BasePage):
+    def submit_form(self):
+        self.browser.select_form(name="formRoutage")
+        self.browser.submit(nologin=True)
+
+
+class InvestmentNatixisPage(_BasePage):
+    COL_LABEL = 0
+    COL_QUANTITY = 2
+    COL_UNITVALUE = 3
+    COL_VALUATION = 4
+    def get_investments(self):
+        for line in self.document.xpath('//div[@class="row-fluid table-contrat-supports"]/table/tbody[(@class)]/tr'):
+            cols = line.findall('td')
+
+            inv = Investment()
+            inv.label = self.parser.tocleanstring(cols[self.COL_LABEL]).replace('Cas sans risque ', '')
+            inv.quantity = self.parse_decimal(cols[self.COL_QUANTITY])
+            inv.unitvalue = self.parse_decimal(cols[self.COL_UNITVALUE])
+            inv.valuation = self.parse_decimal(cols[self.COL_VALUATION])
+
+            yield inv
+
+    def parse_decimal(self, string):
+        value = self.parser.tocleanstring(string).replace('Si famille fonds generaux, on affiche un tiret', '').replace('Cas sans risque', '').replace(' ', '')
+        if value == '-':
+            return NotAvailable
+        return Decimal(Transaction.clean_amount(value))
+
+class MessagePage(_BasePage):
+    def skip(self):
+        try:
+            self.browser.select_form(name="leForm")
+        except FormNotFoundError:
+            pass
+        else:
+            self.browser.submit(nologin=True)
+
+class DocumentsPage(BasePage):
+    def get_account_extract(self, acc_id):
+        for td in self.document.xpath('//tbody[@class="ts tms"]/tr'):
+            if CleanText().filter(td.xpath('./td[1]')[0]) in acc_id and CleanText().filter(td.xpath('./td[4]')[0]) == "Extrait de compte":
+                self.browser.select_form(predicate=lambda form: form.attrs.get('id', '') == 'myForm')
+                self.browser.set_all_readonly(False)
+                doc_id = self.browser['taskOID']
+                self.browser['token'] = self.build_token(self.browser['token'])
+                self.browser['attribute($SEL_ACTIVE$tbl2)'] = td.xpath('./@id')[0].split('_', 1)[1]
+                self.browser['dialogActionPerformed'] = "TELECHARGER"
+                self.browser['attribute($SEL_$lst1_hidden)'] = 'lst1$listeTypeDoc$listeTypeDocSelectionne$'
+                self.browser['attribute($SEL_$lst2_hidden)'] = 'lst2$listeCompteRecherche$listeCompteSelect$'
+                self.browser['dateDebut'] = '23/09/2014'
+                self.browser['dateFin'] = '23/09/2015'
+                self.browser['attribute($SEL_$tbl2_hidden)'] = 'tbl2$listeDocumentSelectionne$'
+                self.browser['attribute($SEL_ACTIVE$tbl2_hidden)'] = 'tbl2$documentSelectionne$'
+                self.browser.submit()
+                return doc_id
+        return False
+
+class PostDocument(BasePage):
+    def get_doc_id(self):
+        for script in self.document.xpath('//script'):
+            if script.text is None:
+                continue
+
+            m = re.search("DocumentUtils\.download\('(.*)'\)", script.text)
+            if m:
+                return m.group(1)
+
+        return None
+
+class ExtractPdf(_BasePage):
+    iban_regexp= r'\(IBAN \) Tj(?:.*[\n\r]){5}\((\w{4})\)(?:.*[\n\r]){10}\((\d{4})\)(?:.*[\n\r]){10}\((\d{4})\)(?:.*[\n\r]){10}\((\d{4})\)(?:.*[\n\r]){10}\((\d{4})\)(?:.*[\n\r]){10}\((\d{4})\)(?:.*[\n\r]){10}\((\d{3})\)'
+    def __init__(self, *args, **kwargs):
+        _BasePage.__init__(self, *args, **kwargs)
+        try:
+            self._pdf = decompress_pdf(self.document)
+        except OSError as e:
+            raise UserError(u'Make sure mupdf-tools is installed (%s)' % e)
+
+    def get_iban(self):
+        res = re.findall(self.iban_regexp, self._pdf)
+        if res and len(res) == 1:
+            return u''.join(res[0])
+        return None
