@@ -22,18 +22,25 @@
 from .account_history import AccountHistory
 from .card_history import CardHistory
 from .accounts_list import AccountsList
-from .login import LoginPage, UpdateInfoPage
-
+from .login import LoginPage, ProfilIncomplete, UpdateInfoPage, NewWebsitePage
 from .two_authentication import AuthenticationPage
+from .investment import AccountMarket, AccountLifeInsurance, InvestmentDetail
+from .lifeinsurance_history import LifeInsuranceHistory
 
 
 class AccountPrelevement(AccountsList):
     pass
 
 __all__ = ['LoginPage',
+           'ProfilIncomplete',
            'AccountsList',
            'AccountHistory',
            'CardHistory',
            'UpdateInfoPage',
            'AuthenticationPage',
+           'AccountMarket',
+           'AccountLifeInsurance',
+           'InvestmentDetail',
+           'LifeInsuranceHistory',
+           'NewWebsitePage',
           ]
