@@ -28,10 +28,14 @@ from weboob.deprecated.mech import ClientForm
 from weboob.tools.ordereddict import OrderedDict
 from weboob.deprecated.browser import Page, BrokenPageError, BrowserUnavailable, BrowserIncorrectPassword
 from weboob.capabilities import NotAvailable
-from weboob.capabilities.bank import Account
+from weboob.capabilities.bank import Account, Investment
 from weboob.tools.capabilities.bank.transactions import FrenchTransaction
 
 
+class GarbagePage(Page):
+    pass
+
+
 class _LogoutPage(Page):
     def on_loaded(self):
         try:
@@ -123,10 +127,21 @@ class IndexPage(Page):
                      u'Compte Courant':             Account.TYPE_CHECKING,
                      u'Mes comptes':                Account.TYPE_CHECKING,
                      u'Mon épargne':                Account.TYPE_SAVINGS,
+                     u'Mes autres comptes':         Account.TYPE_SAVINGS,
+                     u'Compte Epargne et DAT':      Account.TYPE_SAVINGS,
+                     u'Plan et Contrat d\'Epargne': Account.TYPE_SAVINGS,
+                     u'Titres':                     Account.TYPE_MARKET,
+                     u'Compte titres':              Account.TYPE_MARKET,
                     }
 
+    def on_loaded(self):
+        # This page is sometimes an useless step to the market website.
+        bourse_link = self.document.xpath(u'//div[@id="MM_COMPTE_TITRE_pnlbourseoic"]//a[contains(text(), "Accédez à la consultation")]')
+        if len(bourse_link) == 1:
+            self.browser.location(bourse_link[0].attrib['href'])
+
     def _get_account_info(self, a):
-        m = re.search("PostBack(Options)?\([\"'][^\"']+[\"'],\s*['\"]HISTORIQUE_([\d\w&]+)['\"]", a.attrib.get('href', ''))
+        m = re.search("PostBack(Options)?\([\"'][^\"']+[\"'],\s*['\"]([HISTORIQUE_\w|SYNTHESE_ASSURANCE_CNP|BOURSE|COMPTE_TITRE][\d\w&]+)?['\"]", a.attrib.get('href', ''))
         if m is None:
             return None
         else:
@@ -134,7 +149,21 @@ class IndexPage(Page):
             # and is necessary for navigation.
             link = m.group(2)
             parts = link.split('&')
-            return {'type': parts[0], 'id': parts[1], 'link': link}
+            info = {}
+            if len(parts) > 1:
+                info['type'] = parts[0]
+                info['id'] = parts[1]
+            else:
+                id = re.search("([\d]+)", a.attrib.get('title'))
+                info['type'] = link
+                info['id'] = id.group(1)
+            if info['type'] == 'SYNTHESE_ASSURANCE_CNP':
+                info['acc_type'] = Account.TYPE_LIFE_INSURANCE
+            if info['type'] in ('BOURSE', 'COMPTE_TITRE'):
+                info['acc_type'] = Account.TYPE_MARKET
+            info['link'] = link
+            return info
+
 
     def _add_account(self, accounts, link, label, account_type, balance):
         info = self._get_account_info(link)
@@ -144,14 +173,15 @@ class IndexPage(Page):
 
         account = Account()
         account.id = info['id']
+        account.iban = u'FR76' + info['id']
         account._info = info
         account.label = label
-        account.type = account_type
-        account.balance = Decimal(FrenchTransaction.clean_amount(balance))
+        account.type = info['acc_type'] if 'acc_type' in info else account_type
+        account.balance = Decimal(FrenchTransaction.clean_amount(balance)) if balance else self.get_balance(account)
         account.currency = account.get_currency(balance)
         account._card_links = []
 
-        if account._info['type'] == 'CB' and account.id in accounts:
+        if account._info['type'] == 'HISTORIQUE_CB' and account.id in accounts:
             a = accounts[account.id]
             if not a.coming:
                 a.coming = Decimal('0.0')
@@ -161,6 +191,19 @@ class IndexPage(Page):
 
         accounts[account.id] = account
 
+    def get_balance(self, account):
+        if not account.type == Account.TYPE_LIFE_INSURANCE:
+            return NotAvailable
+        self.go_history(account._info)
+        balance = self.browser.page.document.xpath('.//tr[td[contains(text(), ' + account.id + ')]]/td[contains(@class, "somme")]')
+        if len(balance) > 0:
+            balance = self.parser.tocleanstring(balance[0])
+            balance = Decimal(FrenchTransaction.clean_amount(balance)) if balance != u'' else NotAvailable
+        else:
+            balance = NotAvailable
+        self.go_list()
+        return balance
+
     def get_list(self):
         accounts = OrderedDict()
 
@@ -170,28 +213,23 @@ class IndexPage(Page):
             for tr in table.xpath('./tr'):
                 tds = tr.findall('td')
                 if tr.attrib.get('class', '') == 'DataGridHeader':
-                    account_type = self.ACCOUNT_TYPES.get(tds[1].text.strip(), Account.TYPE_UNKNOWN)
+                    account_type = self.ACCOUNT_TYPES.get(tds[1].text.strip()) or\
+                                   self.ACCOUNT_TYPES.get(self.parser.tocleanstring(tds[2])) or\
+                                   self.ACCOUNT_TYPES.get(self.parser.tocleanstring(tds[3]), Account.TYPE_UNKNOWN)
                 else:
-                    label = ''
-                    i = 1
-                    a = None
-                    while label == '' and i < len(tds):
-                        a = tds[i].find('a')
-                        if a is None:
-                            continue
-
-                        label = self.parser.tocleanstring(a)
-                        i += 1
-
-                    balance = ''
-                    i = -1
-                    while balance == '' and i > -len(tds):
-                        try:
-                            balance = self.parser.tocleanstring(tds[i].xpath('./a')[0])
-                        except KeyError:
-                            balance = u''.join([txt.strip() for txt in tds[i].itertext()])
-                        i -= 1
-                    self._add_account(accounts, a, label, account_type, balance)
+                    # On the same row, there are many accounts (for example a
+                    # check accound and a card one).
+                    if len(tds) > 4:
+                        for i, a in enumerate(tds[2].xpath('./a')):
+                            label = self.parser.tocleanstring(a)
+                            balance = self.parser.tocleanstring(tds[-2].xpath('./a')[i])
+                            self._add_account(accounts, a, label, account_type, balance)
+                    # Only 4 tds on banque de la reunion website.
+                    elif len(tds) == 4:
+                        for i, a in enumerate(tds[1].xpath('./a')):
+                            label = self.parser.tocleanstring(a)
+                            balance = self.parser.tocleanstring(tds[-1].xpath('./a')[i])
+                            self._add_account(accounts, a, label, account_type, balance)
 
         if len(accounts) == 0:
             # New website
@@ -247,7 +285,7 @@ class IndexPage(Page):
         self.browser.select_form(name='main')
         self.browser.set_all_readonly(False)
         self.browser['__EVENTTARGET'] = 'MM$SYNTHESE'
-        self.browser['__EVENTARGUMENT'] = 'HISTORIQUE_%s' % info['link']
+        self.browser['__EVENTARGUMENT'] = info['link']
         try:
             self.browser['MM$m_CH$IsMsgInit'] = '0'
         except ControlNotFoundError:
@@ -290,7 +328,7 @@ class IndexPage(Page):
             if len(detail) > 0:
                 detail[0].drop_tree()
 
-            t = Transaction(i)
+            t = Transaction()
 
             date = u''.join([txt.strip() for txt in tds[i+0].itertext()])
             raw = u' '.join([txt.strip() for txt in tds[i+1].itertext()])
@@ -298,7 +336,7 @@ class IndexPage(Page):
             credit = u''.join([txt.strip() for txt in tds[-1].itertext()])
 
             t.parse(date, re.sub(r'[ ]+', ' ', raw))
-            if t.date is NotAvailable:
+            if t.date is NotAvailable or 'Tot Dif' in t.raw:
                 continue
             t.set_amount(credit, debit)
             yield t
@@ -337,3 +375,101 @@ class IndexPage(Page):
         self.browser.submit()
 
         return True
+
+    def go_life_insurance(self, account):
+        link = self.document.xpath('//table[@summary="Mes contrats d\'assurance vie"]/tbody/tr[td[contains(text(), ' + account.id + ') ]]//a')[0]
+        m = re.search("PostBack(Options)?\([\"'][^\"']+[\"'],\s*['\"](REDIR_ASS_VIE[\d\w&]+)?['\"]", link.attrib.get('href', ''))
+        if m is not None:
+            self.browser.select_form(name='main')
+            self.browser.set_all_readonly(False)
+            self.browser['__EVENTTARGET'] = 'MM$SYNTHESE_ASSURANCE_CNP'
+            self.browser['__EVENTARGUMENT'] = m.group(2)
+            try:
+                self.browser['MM$m_CH$IsMsgInit'] = '0'
+            except ControlNotFoundError:
+                # Not available on new website.
+                pass
+            self.browser.controls.append(ClientForm.TextControl('text', 'm_ScriptManager', {'value': ''}))
+            self.browser['m_ScriptManager'] = 'MM$m_UpdatePanel|MM$SYNTHESE'
+            try:
+                self.browser.controls.remove(self.browser.find_control(name='Cartridge$imgbtnMessagerie', type='image'))
+                self.browser.controls.remove(self.browser.find_control(name='MM$m_CH$ButtonImageFondMessagerie', type='image'))
+                self.browser.controls.remove(self.browser.find_control(name='MM$m_CH$ButtonImageMessagerie', type='image'))
+            except ControlNotFoundError:
+                pass
+            self.browser.submit()
+
+
+class MarketPage(Page):
+    def is_error(self):
+        try:
+            return self.document.xpath('//caption')[0].text == "Erreur"
+        except IndexError:
+            return False
+
+
+    def parse_decimal(self, td):
+        value = self.parser.tocleanstring(td)
+        if value and value != '-':
+            return Decimal(FrenchTransaction.clean_amount(value))
+        else:
+            return NotAvailable
+
+    def submit(self):
+        self.browser.select_form(nr=1)
+        self.browser.submit()
+
+    def iter_investment(self):
+        for tbody in self.document.xpath(u'//table[@summary="Contenu du portefeuille valorisé"]/tbody'):
+
+            inv = Investment()
+            inv.label = self.parser.tocleanstring(tbody.xpath('./tr[1]/td[1]/a/span')[0])
+            inv.code = self.parser.tocleanstring(tbody.xpath('./tr[1]/td[1]/a')[0]).split(' - ')[1]
+            inv.quantity = self.parse_decimal(tbody.xpath('./tr[2]/td[2]')[0])
+            inv.unitvalue = self.parse_decimal(tbody.xpath('./tr[2]/td[3]')[0])
+            inv.unitprice = self.parse_decimal(tbody.xpath('./tr[2]/td[5]')[0])
+            inv.valuation = self.parse_decimal(tbody.xpath('./tr[2]/td[4]')[0])
+            inv.diff = self.parse_decimal(tbody.xpath('./tr[2]/td[7]')[0])
+
+            yield inv
+
+    def get_valuation_diff(self, account):
+        valuation_diff = re.sub(r'\(.*\)', '', self.document.xpath(u'//td[contains(text(), "values latentes")]/following-sibling::*[1]')[0].text)
+        account.valuation_diff = Decimal(FrenchTransaction.clean_amount(valuation_diff))
+
+
+class LifeInsurance(MarketPage):
+    def get_cons_repart(self):
+        return self.document.xpath('//tr[@id="sousMenuConsultation3"]/td/div/a')[0].attrib['href']
+
+    def iter_investment(self):
+        for tr in self.document.xpath(u'//table[@class="boursedetail"]/tr[@class and not(@class="total")]'):
+
+            inv = Investment()
+            libelle = self.parser.tocleanstring(tr.xpath('./td[1]')[0]).split(' ')
+            inv.label, inv.code = self.split_label_code(libelle)
+            diff = self.parse_decimal(tr.xpath('./td[6]')[0])
+            inv.quantity = self.parse_decimal(tr.xpath('./td[2]')[0])
+            inv.unitvalue = self.parse_decimal(tr.xpath('./td[3]')[0])
+            inv.unitprice = self.calc(inv.unitvalue, diff)
+            inv.valuation = self.parse_decimal(tr.xpath('./td[5]')[0])
+            inv.diff = self.get_diff(inv.valuation, self.calc(inv.valuation, diff))
+
+            yield inv
+
+    def calc(self, value, diff):
+        if value is NotAvailable or diff is NotAvailable:
+            return NotAvailable
+        return Decimal(value) / (1 + Decimal(diff)/100)
+
+    def get_diff(self, valuation, calc):
+        if valuation is NotAvailable or calc is NotAvailable:
+            return NotAvailable
+        return valuation - calc
+
+    def split_label_code(self, libelle):
+        m = re.search('FR\d+', libelle[-1])
+        if m:
+            return ' '.join(libelle[:-1]), libelle[-1]
+        else:
+            return ' '.join(libelle), NotAvailable
