@@ -18,16 +18,17 @@
 # along with weboob. If not, see <http://www.gnu.org/licenses/>.
 
 
-from .accounts_list import AccountsList
+from .accounts_list import AccountsList, TitreDetails
 from .login import LoginPage, StopPage
 from .transfer import TransferPage, TransferConfirmPage
 from .bills import BillsPage
-from .titre import TitrePage, TitreHistory
+from .titre import NetissimaPage, TitrePage, TitreHistory, TitreValuePage
 
 
 class AccountPrelevement(AccountsList):
     pass
 
-__all__ = ['AccountsList', 'LoginPage',
+__all__ = ['AccountsList', 'LoginPage', 'NetissimaPage','TitreDetails',
            'AccountPrelevement', 'TransferPage', 'TransferConfirmPage',
-           'BillsPage', 'StopPage', 'TitrePage', 'TitreHistory']
+           'BillsPage', 'StopPage', 'TitrePage', 'TitreHistory',
+           'TitreValuePage']
