@@ -18,9 +18,9 @@
 # along with weboob. If not, see <http://www.gnu.org/licenses/>.
 
 
-from weboob.capabilities.weather import CapWeather
+from weboob.capabilities.weather import CapWeather, CityNotFound
 from weboob.tools.backend import Module
-
+from weboob.capabilities.base import find_object
 from .browser import MeteofranceBrowser
 
 
@@ -31,16 +31,27 @@ class MeteofranceModule(Module, CapWeather):
     NAME = 'meteofrance'
     MAINTAINER = u'Cedric Defortis'
     EMAIL = 'cedric@aiur.fr'
-    VERSION = '1.0'
+    VERSION = '1.1'
     DESCRIPTION = 'Get forecasts from the MeteoFrance website'
     LICENSE = 'AGPLv3+'
     BROWSER = MeteofranceBrowser
 
     def get_current(self, city_id):
-        return self.browser.get_current(city_id)
+        return self.browser.get_current(self.get_city(city_id))
 
     def iter_forecast(self, city_id):
-        return self.browser.iter_forecast(city_id)
+        return self.browser.iter_forecast(self.get_city(city_id))
 
     def iter_city_search(self, pattern):
         return self.browser.iter_city_search(pattern)
+
+    def get_city(self, _id):
+        cities = list(self.iter_city_search(_id))
+
+        if len(cities) == 0:
+            raise CityNotFound()
+
+        try:
+            return find_object(cities, id=_id, error=CityNotFound)
+        except CityNotFound:
+            return cities[0]
