@@ -26,7 +26,7 @@ from weboob.applications.suboob.suboob import SubtitleInfoFormatter, SubtitleLis
 from weboob.capabilities.torrent import CapTorrent, MagnetOnly
 from weboob.capabilities.cinema import CapCinema
 from weboob.capabilities.subtitle import CapSubtitle
-from weboob.capabilities.base import empty
+from weboob.capabilities.base import empty, NotAvailable
 from weboob.tools.application.repl import ReplApplication, defaultcount
 from weboob.tools.application.formatters.iformatter import IFormatter, PrettyFormatter
 from weboob.core import CallErrors
@@ -59,8 +59,8 @@ class MovieInfoFormatter(IFormatter):
             result += '\n%sRelated persons%s\n' % (self.BOLD, self.NC)
             for role, lpersons in obj.roles.items():
                 result += ' -- %s\n' % role
-                for name in lpersons:
-                    result += '   * %s\n' % name
+                for person in lpersons:
+                    result += '   * %s\n' % person[1]
         if not empty(obj.other_titles):
             result += '\n%sOther titles%s\n' % (self.BOLD, self.NC)
             for t in obj.other_titles:
@@ -143,7 +143,7 @@ class PersonInfoFormatter(IFormatter):
             for role, lmovies in obj.roles.items():
                 result += ' -- %s\n' % role
                 for movie in lmovies:
-                    result += '   * %s\n' % movie
+                    result += '   * %s\n' % movie[1]
         if not empty(obj.short_biography):
             result += '\n%sShort biography%s\n' % (self.BOLD, self.NC)
             result += '%s' % obj.short_biography
@@ -176,8 +176,8 @@ class PersonBiographyFormatter(PrettyFormatter):
 
 class Cineoob(ReplApplication):
     APPNAME = 'cineoob'
-    VERSION = '1.0'
-    COPYRIGHT = 'Copyright(C) 2013 Julien Veyssier'
+    VERSION = '1.1'
+    COPYRIGHT = 'Copyright(C) 2013-YEAR Julien Veyssier'
     DESCRIPTION = "Console application allowing to search for movies and persons on various cinema databases " \
                   ", list persons related to a movie, list movies related to a person and list common movies " \
                   "of two persons."
@@ -248,10 +248,31 @@ class Cineoob(ReplApplication):
             lid2.append(id)
         self.options.count = initial_count
         inter = list(set(lid1) & set(lid2))
+
+        chrono_list = []
         for common in inter:
             movie = self.get_object(common, 'get_movie', caps=CapCinema)
+            role1 = movie.get_roles_by_person_id(person1.id)
+            if not role1:
+                role1 = movie.get_roles_by_person_name(person1.name)
+            role2 = movie.get_roles_by_person_id(person2.id)
+            if not role2:
+                role2 = movie.get_roles_by_person_name(person2.name)
+
+            if (movie.release_date != NotAvailable):
+                year = movie.release_date.year
+            else:
+                year = '????'
+            movie.short_description = '(%s) %s as %s ; %s as %s'%(year, person1.name, ', '.join(role1), person2.name, ', '.join(role2))
             if movie:
-                self.cached_format(movie)
+                i = 0
+                while (i<len(chrono_list) and movie.release_date != NotAvailable and
+                      (chrono_list[i].release_date == NotAvailable or year > chrono_list[i].release_date.year)):
+                    i += 1
+                chrono_list.insert(i, movie)
+
+        for movie in chrono_list:
+            self.cached_format(movie)
 
     def do_persons_in_common(self, line):
         """
@@ -283,6 +304,13 @@ class Cineoob(ReplApplication):
         inter = list(set(lid1) & set(lid2))
         for common in inter:
             person = self.get_object(common, 'get_person', caps=CapCinema)
+            role1 = person.get_roles_by_movie_id(movie1.id)
+            if not role1:
+                role1 = person.get_roles_by_movie_title(movie1.original_title)
+            role2 = person.get_roles_by_movie_id(movie2.id)
+            if not role2:
+                role2 = person.get_roles_by_movie_title(movie2.original_title)
+            person.short_description = '%s in %s ; %s in %s'%(', '.join(role1), movie1.original_title, ', '.join(role2), movie2.original_title)
             self.cached_format(person)
 
     def do_info_movie(self, id):
@@ -475,7 +503,7 @@ class Cineoob(ReplApplication):
                         print(buf)
                     else:
                         try:
-                            with open(dest, 'w') as f:
+                            with open(dest, 'wb') as f:
                                 f.write(buf)
                         except IOError as e:
                             print('Unable to write .torrent in "%s": %s' % (dest, e), file=self.stderr)
