@@ -23,7 +23,7 @@ from __future__ import print_function
 import logging
 import optparse
 from optparse import OptionGroup, OptionParser
-import locale
+from datetime import datetime
 import os
 import sys
 import warnings
@@ -34,7 +34,7 @@ from weboob.core.backendscfg import BackendsConfig
 from weboob.tools.config.iconfig import ConfigError
 from weboob.exceptions import FormFieldConversionWarning
 from weboob.tools.log import createColoredFormatter, getLogger, DEBUG_FILTERS, settings as log_settings
-from weboob.tools.misc import to_unicode
+from weboob.tools.misc import to_unicode, guess_encoding
 from .results import ResultsConditionError
 
 __all__ = ['Application']
@@ -174,13 +174,7 @@ class Application(object):
         self._is_default_count = True
 
     def guess_encoding(self, stdio=None):
-        if stdio is None:
-            stdio = self.stdout
-        encoding = stdio.encoding or locale.getpreferredencoding()
-        # ASCII or ANSII is most likely a user mistake
-        if not encoding or encoding.lower() == 'ascii' or encoding.lower().startswith('ansi'):
-            encoding = 'UTF-8'
-        return encoding
+        return guess_encoding(stdio or self.stdout)
 
     def deinit(self):
         self.weboob.want_stop()
@@ -260,7 +254,8 @@ class Application(object):
         version = None
         if self.VERSION:
             if self.COPYRIGHT:
-                version = '%s v%s %s' % (self.APPNAME, self.VERSION, self.COPYRIGHT)
+                copyright = self.COPYRIGHT.replace('YEAR', '%d' % datetime.today().year)
+                version = '%s v%s %s' % (self.APPNAME, self.VERSION, copyright)
             else:
                 version = '%s v%s' % (self.APPNAME, self.VERSION)
         return version
