@@ -24,6 +24,7 @@ from cmd import Cmd
 import logging
 import re
 from optparse import OptionGroup, OptionParser, IndentedHelpFormatter
+from datetime import datetime
 import os
 
 from weboob.capabilities.base import FieldNotFound, BaseObject, UserError
@@ -113,9 +114,10 @@ class ReplApplication(Cmd, ConsoleApplication):
         Cmd.__init__(self)
         ConsoleApplication.__init__(self, ReplOptionParser(self.SYNOPSIS, version=self._get_optparse_version()))
 
+        copyright = self.COPYRIGHT.replace('YEAR', '%d' % datetime.today().year).encode(self.encoding)
         self.intro = '\n'.join(('Welcome to %s%s%s v%s' % (self.BOLD, self.APPNAME, self.NC, self.VERSION),
                                 '',
-                                self.COPYRIGHT.encode(self.encoding),
+                                copyright,
                                 'This program is free software: you can redistribute it and/or modify',
                                 'it under the terms of the GNU Affero General Public License as published by',
                                 'the Free Software Foundation, either version 3 of the License, or',
@@ -340,8 +342,9 @@ class ReplApplication(Cmd, ConsoleApplication):
         if backends is None:
             kwargs['backends'] = []
             for backend in self.enabled_backends:
-                actual_function = getattr(backend, function, None)
-                if actual_function is not None and callable(actual_function):
+                actual_function = getattr(backend, function, None) if isinstance(function, basestring) else function
+
+                if callable(actual_function):
                     kwargs['backends'].append(backend)
         else:
             kwargs['backends'] = backends
@@ -462,11 +465,11 @@ class ReplApplication(Cmd, ConsoleApplication):
     def path_completer(self, arg):
         dirname = os.path.dirname(arg)
         try:
-            childs = os.listdir(dirname or '.')
+            children = os.listdir(dirname or '.')
         except OSError:
             return ()
         l = []
-        for child in childs:
+        for child in children:
             path = os.path.join(dirname, child)
             if os.path.isdir(path):
                 child += '/'
@@ -709,7 +712,7 @@ class ReplApplication(Cmd, ConsoleApplication):
                 print('Disabled: %s' % ', '.join(disabled_backends_names))
         elif action == 'add':
             for name in given_backend_names:
-                instname = self.add_backend(name)
+                instname = self.add_backend(name, name)
                 if instname:
                     self.load_backends(names=[instname])
         elif action == 'register':
